# -*- coding: utf-8 -*-

"""Tools dedicated to proteins"""

#------------------------Dictionnary-----------------

"""Dictionary of 'POSSIBLE SPACE-GROUPS FOR PROTEIN CRYSTALS [SPACE GROUP NUMBER,SYMBOL]' """
sg_nb_to_symbol = {1:"P1", 3:"P2",4:"P21",5:"C2",16:"P222",17:"P2221",18:"P21212",
19:"P212121",20:"C2221",21:"C222",22:"F222",23:"I222",24:"I212121",
75:"P4",76:"P41",77:"P42",78:"P43",79:"I4", 80:"I41", 89:"P422",90:"P4212",
91:"P4122",92:"P41212", 93:"P4222",94:"P42212",95:"P4322",96:"P43212",
97:"I422",98:"I4122", 143:"P3",144:"P31",145:"P32",146:"R3", 149:"P312",150:"P321",
151:"P3112",152:"P3121",153:"P3212",154:"P3221",155:"R32", 168:"P6",
169:"P61",170:"P65",171:"P62",172:"P64",173:"P63",177:"P622",178:"P6122",179:"P6522",
180:"P6222",181:"P6422",182:"P6322",195:"P23",196:"F23",197:"I23",198:"P213",199:"I213",
207:"P432",208:"P4232",209:"F432",210:"F4132",211:"I432",
212:"P4332",213:"P4132",214:"I4132"}

#--------------Functions---------------------------

def sg_from_pdb(fichier_pdb):
    """
    Get space group from a pdb file.
    :param fichier_pdb: pdb file
    :return: space group, if not found return ""
    """
    sg = ""     # space group
    with open(fichier_pdb, 'r') as f_pdb:
        cryst1_OK = False
        # éviter l'erreur : UnicodeDecodeError: 'utf-8' codec can't decode byte 0xa8 in position 37: invalid start byte
        try:
            for l in f_pdb.readlines():  # on lit les lignes du fichier une par une
                if l.split()[0] == "CRYST1":
                    cryst1_OK = True
                    space_group = "".join(l.split()[7:])
                    sg = clean_sg_name(space_group)
                    if not sg:
                        # enlever le dernier chiffre de la ligne qui est parfois là
                        space_group = space_group[:-1]
                        sg = clean_sg_name(space_group)
            if not cryst1_OK:
                print("No row starting with 'CRYST1' in the pdb file, no space group information ...")
        except UnicodeDecodeError as e:
            print(e)
    return sg

def find_key(dict, v):
    """
    Returns the key of a dictionary from a value
    :param dict: dictionary
    :param v: value
    :return: the corresponding key, if doesn't exist return ''
    """
    for k, val in dict.items():
        if v == val:
            return k
    return ""

def clean_sg_name(sg):
    """
    To clean space group name so as to find it in the dictionnary of space groups
    :param sg: space group to check
    :return: cleaned space group or "" if not found
    """
    # enlever les espaces
    #sg.replace(" ", "")
    # vérifier que pas une autre écriture
    sg_pbm = {'P121': 'P2', 'P1211': 'P21', 'C121': 'C2'}  # dictionnaire de correspondances
    if sg in list(sg_pbm):  # liste des clés du dictionnaire
        sg = sg_pbm[sg]
    # vérifier que bien dans la liste des sg
    if sg in sg_nb_to_symbol.values():
        return sg
    else:
        return ""

